@@ -3,24 +3,34 @@
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 #ifndef VBOX_INCLUDED_log_h
@@ -66,6 +76,8 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_AUDIO_MIXER,
     /** Audio mixer buffer group. */
     LOG_GROUP_AUDIO_MIXER_BUFFER,
+    /** Audio test group. */
+    LOG_GROUP_AUDIO_TEST,
     /** Auto-logon group. */
     LOG_GROUP_AUTOLOGON,
     /** CFGM group. */
@@ -96,12 +108,16 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_DEV_BUSLOGIC,
     /** DMA Controller group. */
     LOG_GROUP_DEV_DMA,
+    /** NS DP8390 Ethernet Device group. */
+    LOG_GROUP_DEV_DP8390,
     /** Gigabit Ethernet Device group. */
     LOG_GROUP_DEV_E1000,
     /** Extensible Firmware Interface Device group. */
     LOG_GROUP_DEV_EFI,
     /** USB EHCI Device group. */
     LOG_GROUP_DEV_EHCI,
+    /** 3C501 Ethernet Device group. */
+    LOG_GROUP_DEV_ELNK,
     /** Floppy Controller Device group. */
     LOG_GROUP_DEV_FDC,
     /** Flash Device group. */
@@ -120,6 +136,8 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_DEV_INIP,
     /** I/O APIC Device group. */
     LOG_GROUP_DEV_IOAPIC,
+    /** IOMMU Device group. */
+    LOG_GROUP_DEV_IOMMU,
     /** KeyBoard Controller Device group. */
     LOG_GROUP_DEV_KBD,
     /** Low Pin Count Device group. */
@@ -148,6 +166,8 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_DEV_PIC,
     /** PIT Device group. */
     LOG_GROUP_DEV_PIT,
+    /** QEMU firmware config Device group. */
+    LOG_GROUP_DEV_QEMUFWCFG,
     /** RTC Device group. */
     LOG_GROUP_DEV_RTC,
     /** SB16 Device group. */
@@ -156,6 +176,8 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_DEV_SERIAL,
     /** System Management Controller Device group. */
     LOG_GROUP_DEV_SMC,
+    /** Trusted Platform Module Device group. */
+    LOG_GROUP_DEV_TPM,
     /** VGA Device group. */
     LOG_GROUP_DEV_VGA,
     /** Virtio PCI Device group. */
@@ -184,6 +206,8 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_DRV_BLOCK,
     /** Char driver group. */
     LOG_GROUP_DRV_CHAR,
+    /** Cloud tunnel driver group. */
+    LOG_GROUP_DRV_CTUN,
     /** Disk integrity driver group. */
     LOG_GROUP_DRV_DISK_INTEGRITY,
     /** Video Display driver group. */
@@ -228,6 +252,10 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_DRV_SCSIHOST,
     /** TCP socket stream driver group. */
     LOG_GROUP_DRV_TCP,
+    /** Trusted Platform Module Emulation driver group. */
+    LOG_GROUP_DRV_TPM_EMU,
+    /** Trusted Platform Module Host driver group. */
+    LOG_GROUP_DRV_TPM_HOST,
     /** Async transport driver group */
     LOG_GROUP_DRV_TRANSPORT_ASYNC,
     /** TUN network transport driver group */
@@ -242,6 +270,8 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_DRV_VBOXHDD,
     /** VBox HDD container media driver group. */
     LOG_GROUP_DRV_VD,
+    /** The VMNET networking driver group. */
+    LOG_GROUP_DRV_VMNET,
     /** VRDE audio driver group. */
     LOG_GROUP_DRV_VRDE_AUDIO,
     /** Virtual Switch transport driver group */
@@ -272,6 +302,10 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_HM,
     /** IEM group. */
     LOG_GROUP_IEM,
+    /** IEM AMD-V group. */
+    LOG_GROUP_IEM_SVM,
+    /** IEM VT-x group. */
+    LOG_GROUP_IEM_VMX,
     /** I/O buffer management group. */
     LOG_GROUP_IOBUFMGMT,
     /** IOM group. */
@@ -348,6 +382,10 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_MAIN_APPLIANCE,
     /** Main group, IAudioAdapter. */
     LOG_GROUP_MAIN_AUDIOADAPTER,
+    /** Main group, IAudioDevice. */
+    LOG_GROUP_MAIN_AUDIODEVICE,
+    /** Main group, IAudioSettings. */
+    LOG_GROUP_MAIN_AUDIOSETTINGS,
     /** Main group, IBandwidthControl. */
     LOG_GROUP_MAIN_BANDWIDTHCONTROL,
     /** Main group, IBandwidthGroup. */
@@ -362,6 +400,8 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_MAIN_CHOICEFORMVALUE,
     /** Main group, ICloudClient. */
     LOG_GROUP_MAIN_CLOUDCLIENT,
+    /** Main group, ICloudMachine. */
+    LOG_GROUP_MAIN_CLOUDMACHINE,
     /** Main group, ICloudNetwork. */
     LOG_GROUP_MAIN_CLOUDNETWORK,
     /** Main group, ICloudNetworkEnvironmentInfo */
@@ -370,12 +410,18 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_MAIN_CLOUDNETWORKGATEWAYINFO,
     /** Main group, ICloudProfile. */
     LOG_GROUP_MAIN_CLOUDPROFILE,
+    /** Main group, ICloudProfileChangedEvent. */
+    LOG_GROUP_MAIN_CLOUDPROFILECHANGEDEVENT,
+    /** Main group, ICloudProfileRegisteredEvent. */
+    LOG_GROUP_MAIN_CLOUDPROFILEREGISTEREDEVENT,
     /** Main group, ICloudProvider. */
     LOG_GROUP_MAIN_CLOUDPROVIDER,
     /** Main group, ICloudProviderManager. */
     LOG_GROUP_MAIN_CLOUDPROVIDERMANAGER,
     /** Main group, IConsole. */
     LOG_GROUP_MAIN_CONSOLE,
+    /** Main group, ICPUProfile. */
+    LOG_GROUP_MAIN_CPUPROFILE,
     /** Main group, IDataModel. */
     LOG_GROUP_MAIN_DATAMODEL,
     /** Main group, IDataStream. */
@@ -432,12 +478,16 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_MAIN_FRAMEBUFFER,
     /** Main group, IFramebufferOverlay. */
     LOG_GROUP_MAIN_FRAMEBUFFEROVERLAY,
+    /** Main group, IFsInfo. */
+    LOG_GROUP_MAIN_FSINFO,
     /** Main group, IFsObjInfo. */
     LOG_GROUP_MAIN_FSOBJINFO,
     /** Main group, IGraphicsAdapter. */
     LOG_GROUP_MAIN_GRAPHICSADAPTER,
     /** Main group, IGuest. */
     LOG_GROUP_MAIN_GUEST,
+    /** Main group, IGuestDebugControl. */
+    LOG_GROUP_MAIN_GUESTDEBUGCONTROL,
     /** Main group, IGuestDirectory. */
     LOG_GROUP_MAIN_GUESTDIRECTORY,
     /** Main group, IGuestDnDSource. */
@@ -452,6 +502,8 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_MAIN_GUESTFILEEVENT,
     /** Main group, IGuestFileIOEvent. */
     LOG_GROUP_MAIN_GUESTFILEIOEVENT,
+    /** Main group, IGuestFsInfo. */
+    LOG_GROUP_MAIN_GUESTFSINFO,
     /** Main group, IGuestFsObjInfo. */
     LOG_GROUP_MAIN_GUESTFSOBJINFO,
     /** Main group, IGuestOSType. */
@@ -470,8 +522,20 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_MAIN_GUESTSESSIONEVENT,
     /** Main group, IHost. */
     LOG_GROUP_MAIN_HOST,
+    /** Main group, IHostAudioDevice. */
+    LOG_GROUP_MAIN_HOSTAUDIODEVICE,
+    /** Main group, IHostDrive. */
+    LOG_GROUP_MAIN_HOSTDRIVE,
+    /** Main group, IHostDriveList. */
+    LOG_GROUP_MAIN_HOSTDRIVELIST,
+    /** Main group, IHostDrivePartition. */
+    LOG_GROUP_MAIN_HOSTDRIVEPARTITION,
     /** Main group, IHostNetworkInterface. */
     LOG_GROUP_MAIN_HOSTNETWORKINTERFACE,
+    /** Main group, IHostOnlyNetwork. */
+    LOG_GROUP_MAIN_HOSTONLYNETWORK,
+    /** Main group, IHostUpdateAgent. */
+    LOG_GROUP_MAIN_HOSTUPDATEAGENT,
     /** Main group, IHostUSBDevice. */
     LOG_GROUP_MAIN_HOSTUSBDEVICE,
     /** Main group, IHostUSBDeviceFilter. */
@@ -508,6 +572,8 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_MAIN_NATNETWORK,
     /** Main group, INetworkAdapter. */
     LOG_GROUP_MAIN_NETWORKADAPTER,
+    /** Main group, INvramStore. */
+    LOG_GROUP_MAIN_NVRAMSTORE,
     /** Main group, IParallelPort. */
     LOG_GROUP_MAIN_PARALLELPORT,
     /** Main group, IPCIAddress. */
@@ -522,10 +588,14 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_MAIN_PROCESS,
     /** Main group, IProgress. */
     LOG_GROUP_MAIN_PROGRESS,
+    /** Main group, IProgressCreatedEvent. */
+    LOG_GROUP_MAIN_PROGRESSCREATEDEVENT,
     /** Main group, IProgressEvent. */
     LOG_GROUP_MAIN_PROGRESSEVENT,
     /** Main group, IRangedIntegerFormValue. */
     LOG_GROUP_MAIN_RANGEDINTEGERFORMVALUE,
+    /** Main group, IRangedInteger64FormValue. */
+    LOG_GROUP_MAIN_RANGEDINTEGER64FORMVALUE,
     /** Main group, IRecordingScreenSettings. */
     LOG_GROUP_MAIN_RECORDINGSCREENSETTINGS,
     /** Main group, IRecordingSettings. */
@@ -554,8 +624,24 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_MAIN_THREAD_TASK,
     /** Main group, IToken. */
     LOG_GROUP_MAIN_TOKEN,
+    /** Main group, ITrustedPlatformModule. */
+    LOG_GROUP_MAIN_TRUSTEDPLATFORMMODULE,
+    /** Main group, IUefiVariableStore. */
+    LOG_GROUP_MAIN_UEFIVARIABLESTORE,
     /** Main group, IUnattended. */
     LOG_GROUP_MAIN_UNATTENDED,
+    /** Main group, IUpdateAgent. */
+    LOG_GROUP_MAIN_UPDATEAGENT,
+    /** Main group, IUpdateAgentAvailableEvent. */
+    LOG_GROUP_MAIN_UPDATEAGENTAVAILABLEEVENT,
+    /** Main group, IUpdateAgentErrorEvent. */
+    LOG_GROUP_MAIN_UPDATEAGENTERROREVENT,
+    /** Main group, IUpdateAgentEvent. */
+    LOG_GROUP_MAIN_UPDATEAGENTEVENT,
+    /** Main group, IUpdateAgentSettingsChangedEvent. */
+    LOG_GROUP_MAIN_UPDATEAGENTSETTINGSCHANGEDEVENT,
+    /** Main group, IUpdateAgentStateChangedEvent. */
+    LOG_GROUP_MAIN_UPDATEAGENTSTATECHANGEDEVENT,
     /** Main group, IUSBController. */
     LOG_GROUP_MAIN_USBCONTROLLER,
     /** Main group, IUSBDevice. */
@@ -626,6 +712,8 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_PDM_BLK_CACHE,
     /** PDM critical section group. */
     LOG_GROUP_PDM_CRITSECT,
+    /** PDM read/write critical section group. */
+    LOG_GROUP_PDM_CRITSECTRW,
     /** PDM Device group. */
     LOG_GROUP_PDM_DEVICE,
     /** PDM Driver group. */
@@ -650,6 +738,8 @@ typedef enum VBOXLOGGROUP
     LOG_GROUP_PGM_POOL,
     /** PGM shared paging group. */
     LOG_GROUP_PGM_SHARED,
+    /** Audio + video recording. */
+    LOG_GROUP_RECORDING,
     /** REM group. */
     LOG_GROUP_REM,
     /** REM disassembly handler group. */
@@ -767,6 +857,7 @@ typedef enum VBOXLOGGROUP
     "DEFAULT", \
     "AUDIO_MIXER", \
     "AUDIO_MIXER_BUFFER", \
+    "AUDIO_TEST", \
     "AUTOLOGON", \
     "CFGM", \
     "CPUM", \
@@ -782,9 +873,11 @@ typedef enum VBOXLOGGROUP
     "DEV_APIC", \
     "DEV_BUSLOGIC", \
     "DEV_DMA", \
+    "DEV_DP8390", \
     "DEV_E1000", \
     "DEV_EFI", \
     "DEV_EHCI", \
+    "DEV_ELNK", \
     "DEV_FDC", \
     "DEV_FLASH", \
     "DEV_GIM", \
@@ -794,6 +887,7 @@ typedef enum VBOXLOGGROUP
     "DEV_IDE", \
     "DEV_INIP", \
     "DEV_IOAPIC", \
+    "DEV_IOMMU", \
     "DEV_KBD", \
     "DEV_LPC", \
     "DEV_LSILOGICSCSI", \
@@ -808,10 +902,12 @@ typedef enum VBOXLOGGROUP
     "DEV_PCNET", \
     "DEV_PIC", \
     "DEV_PIT", \
+    "DEV_QEMUFWCFG", \
     "DEV_RTC", \
     "DEV_SB16", \
     "DEV_SERIAL", \
     "DEV_SMC", \
+    "DEV_TPM", \
     "DEV_VGA", \
     "DEV_VIRTIO", \
     "DEV_VIRTIO_NET", \
@@ -826,6 +922,7 @@ typedef enum VBOXLOGGROUP
     "DRV_AUDIO", \
     "DRV_BLOCK", \
     "DRV_CHAR", \
+    "DRV_CTUN", \
     "DRV_DISK_INTEGRITY", \
     "DRV_DISPLAY", \
     "DRV_FLOPPY", \
@@ -848,6 +945,8 @@ typedef enum VBOXLOGGROUP
     "DRV_SCSI", \
     "DRV_SCSIHOST", \
     "DRV_TCP", \
+    "DRV_TPM_EMU", \
+    "DRV_TPM_HOST", \
     "DRV_TRANSPORT_ASYNC", \
     "DRV_TUN", \
     "DRV_UDP", \
@@ -855,6 +954,7 @@ typedef enum VBOXLOGGROUP
     "DRV_USBPROXY", \
     "DRV_VBOXHDD", \
     "DRV_VD", \
+    "DRV_VMNET", \
     "DRV_VRDE_AUDIO", \
     "DRV_VSWITCH", \
     "DRV_VUSB", \
@@ -870,6 +970,8 @@ typedef enum VBOXLOGGROUP
     "HGSMI", \
     "HM", \
     "IEM", \
+    "IEM_SVM", \
+    "IEM_VMX", \
     "IOBUFMGMT", \
     "IOM", \
     "IOM_IOPORT", \
@@ -908,6 +1010,8 @@ typedef enum VBOXLOGGROUP
     "MAIN_ADDITIONSFACILITY", \
     "MAIN_APPLIANCE", \
     "MAIN_AUDIOADAPTER", \
+    "MAIN_AUDIODEVICE", \
+    "MAIN_AUDIOSETTINGS", \
     "MAIN_BANDWIDTHCONTROL", \
     "MAIN_BANDWIDTHGROUP", \
     "MAIN_BIOSSETTINGS", \
@@ -915,13 +1019,17 @@ typedef enum VBOXLOGGROUP
     "MAIN_CERTIFICATE", \
     "MAIN_CHOICEFORMVALUE", \
     "MAIN_CLOUDCLIENT", \
+    "MAIN_CLOUDMACHINE", \
     "MAIN_CLOUDNETWORK", \
     "MAIN_CLOUDNETWORKENVIRONMENTINFO", \
     "MAIN_CLOUDNETWORKGATEWAYINFO", \
     "MAIN_CLOUDPROFILE", \
+    "MAIN_CLOUDPROFILECHANGEDEVENT",   \
+    "MAIN_CLOUDPROFILEREGISTEREDEVENT",   \
     "MAIN_CLOUDPROVIDER", \
     "MAIN_CLOUDPROVIDERMANAGER", \
     "MAIN_CONSOLE", \
+    "MAIN_CPUPROFILE", \
     "MAIN_DATAMODEL", \
     "MAIN_DATASTREAM", \
     "MAIN_DHCPCONFIG", \
@@ -950,9 +1058,11 @@ typedef enum VBOXLOGGROUP
     "MAIN_FORMVALUE", \
     "MAIN_FRAMEBUFFER", \
     "MAIN_FRAMEBUFFEROVERLAY", \
+    "MAIN_FSINFO", \
     "MAIN_FSOBJINFO", \
     "MAIN_GRAPHICSADAPTER", \
     "MAIN_GUEST", \
+    "MAIN_GUESTDEBUGCONTROL", \
     "MAIN_GUESTDIRECTORY", \
     "MAIN_GUESTDNDSOURCE", \
     "MAIN_GUESTDNDTARGET", \
@@ -960,6 +1070,7 @@ typedef enum VBOXLOGGROUP
     "MAIN_GUESTFILE", \
     "MAIN_GUESTFILEEVENT", \
     "MAIN_GUESTFILEIOEVENT", \
+    "MAIN_GUESTFSINFO", \
     "MAIN_GUESTFSOBJINFO", \
     "MAIN_GUESTOSTYPE", \
     "MAIN_GUESTPROCESS", \
@@ -969,7 +1080,13 @@ typedef enum VBOXLOGGROUP
     "MAIN_GUESTSESSION", \
     "MAIN_GUESTSESSIONEVENT", \
     "MAIN_HOST", \
+    "MAIN_HOSTAUDIODEVICE", \
+    "MAIN_HOSTDRIVE", \
+    "MAIN_HOSTDRIVELIST", \
+    "MAIN_HOSTDRIVEPARTITION", \
     "MAIN_HOSTNETWORKINTERFACE", \
+    "MAIN_HOSTONLYNETWORK", \
+    "MAIN_HOSTUPDATEAGENT", \
     "MAIN_HOSTUSBDEVICE", \
     "MAIN_HOSTUSBDEVICEFILTER", \
     "MAIN_HOSTVIDEOINPUTDEVICE", \
@@ -988,6 +1105,7 @@ typedef enum VBOXLOGGROUP
     "MAIN_NATENGINE", \
     "MAIN_NATNETWORK", \
     "MAIN_NETWORKADAPTER", \
+    "MAIN_NVRAMSTORE", \
     "MAIN_PARALLELPORT", \
     "MAIN_PCIADDRESS", \
     "MAIN_PCIDEVICEATTACHMENT", \
@@ -995,7 +1113,9 @@ typedef enum VBOXLOGGROUP
     "MAIN_PERFORMANCEMETRIC", \
     "MAIN_PROCESS", \
     "MAIN_PROGRESS", \
+    "MAIN_PROGRESSCREATEDEVENT", \
     "MAIN_PROGRESSEVENT", \
+    "MAIN_RANGEDINTEGER64FORMVALUE", \
     "MAIN_RANGEDINTEGERFORMVALUE", \
     "MAIN_RECORDINGSCREENSETTINGS", \
     "MAIN_RECORDINGSETTINGS", \
@@ -1011,7 +1131,15 @@ typedef enum VBOXLOGGROUP
     "MAIN_SYSTEMPROPERTIES", \
     "MAIN_THREAD_TASK", \
     "MAIN_TOKEN", \
+    "MAIN_TRUSTEDPLATFORMMODULE", \
+    "MAIN_UEFIVARIABLESTORE", \
     "MAIN_UNATTENDED", \
+    "MAIN_UPDATEAGENT", \
+    "MAIN_UPDATEAGENTAVAILABLEEVENT", \
+    "MAIN_UPDATEAGENTERROREVENT", \
+    "MAIN_UPDATEAGENTEVENT", \
+    "MAIN_UPDATEAGENTSETTINGSCHANGEDEVENT", \
+    "MAIN_UPDATEAGENTSTATECHANGEDEVENT", \
     "MAIN_USBCONTROLLER", \
     "MAIN_USBDEVICE", \
     "MAIN_USBDEVICEFILTER", \
@@ -1047,6 +1175,7 @@ typedef enum VBOXLOGGROUP
     "PDM_ASYNC_COMPLETION", \
     "PDM_BLK_CACHE", \
     "PDM_CRITSECT", \
+    "PDM_CRITSECTRW", \
     "PDM_DEVICE", \
     "PDM_DRIVER", \
     "PDM_LDR", \
@@ -1059,6 +1188,7 @@ typedef enum VBOXLOGGROUP
     "PGM_PHYS_ACCESS", \
     "PGM_POOL", \
     "PGM_SHARED", \
+    "RECORDING", \
     "REM", \
     "REM_DISAS", \
     "REM_HANDLER", \
